/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is the Netscape security libraries.
 *
 * The Initial Developer of the Original Code is
 * Netscape Communications Corporation.
 * Portions created by the Initial Developer are Copyright (C) 1994-2000
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *   Dan Kegel <dank@kegel.com>, Google
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

/* TODO: nspr-isze it */

#ifndef __MEMIO_BUFFER_H
#define __MEMIO_BUFFER_H

#ifdef __cplusplus
extern "C" {
#endif 

/*----------------------------------------------------------------------
 Simple circular buffer class.  Size cannot be changed once allocated.
----------------------------------------------------------------------*/

struct memio_buffer {
    int head;     /* where to take next byte out of buf */
    int tail;     /* where to put next byte into buf */
    int bufsize;  /* number of bytes allocated to buf */
    char *buf;
};

/* Allocate a memio_buffer of given size. */
void memio_buffer_new(struct memio_buffer *mb, int size);

/* Deallocate a memio_buffer allocated by memio_buffer_new. */
void memio_buffer_destroy(struct memio_buffer *mb);

/* How many bytes have been put into the buffer */
int memio_buffer_used(const struct memio_buffer *mb);

/* How many bytes can be read out of the buffer without wrapping */
int memio_buffer_used_contig(const struct memio_buffer *mb);

/* How many bytes can still be put into the buffer */
int memio_buffer_unused(const struct memio_buffer *mb);

/* How many bytes can be written into the buffer without wrapping */
int memio_buffer_unused_contig(const struct memio_buffer *mb);

/* Is buffer completely empty? */
int memio_buffer_empty(const struct memio_buffer *mb);

/* Is buffer completely full? */
int memio_buffer_full(const struct memio_buffer *mb);

/* Write n bytes into the buffer.  Returns number of bytes written. */
int memio_buffer_put(struct memio_buffer *mb, const char *buf, int n);

/* Read n bytes from the buffer.  Returns number of bytes read. */
int memio_buffer_get(struct memio_buffer *mb, char *buf, int n);

#define MEMIO_BUFFER_EMPTY -2

/* For output buffers: call this periodically to write buffer contents to a 
 * file/socket. 
 * Returns byte count on success, 
 * -1 on error with errno set,
 * MEMIO_BUFFER_EMPTY on buffer empty.
 */
int memio_buffer_send(struct memio_buffer *mb, int fd);

/* Ditto, but fills input buffers. */
int memio_buffer_recv(struct memio_buffer *mb, int fd);

#ifdef __cplusplus
}
#endif 

#endif
